@@ -27,145 +27,21 @@
 #include <gmodule.h>
 #include <vncdisplay.h>
 
-#include <vinagre/vinagre-debug.h>
 #include <vinagre/vinagre-prefs.h>
 #include <vinagre/vinagre-cache-prefs.h>
+#include <vinagre/vinagre-protocol.h>
 
 #include "vinagre-vnc-plugin.h"
 #include "vinagre-vnc-connection.h"
 #include "vinagre-vnc-tab.h"
-#include "vinagre-vnc-listener-dialog.h"
-#include "vinagre-vnc-listener.h"
 
-#define VINAGRE_VNC_PLUGIN_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), VINAGRE_TYPE_VNC_PLUGIN, VinagreVncPluginPrivate))
-#define WINDOW_DATA_KEY "VinagreVNCPluginWindowData"
-
-VINAGRE_PLUGIN_REGISTER_TYPE(VinagreVncPlugin, vinagre_vnc_plugin)
-
-typedef struct
-{
-  GtkActionGroup *ui_action_group;
-  guint ui_id;
-} WindowData;
-
-typedef struct
-{
-  VinagrePlugin *plugin;
-  VinagreWindow *window;
-} ActionData;
-
-static void
-free_window_data (WindowData *data)
-{
-  g_return_if_fail (data != NULL);
-
-  g_object_unref (data->ui_action_group);
-  g_slice_free (WindowData, data);
-}
-
-static void
-free_action_data (ActionData *data)
-{
-  g_return_if_fail (data != NULL);
-
-  g_slice_free (ActionData, data);
-}
-
-static void
-listening_cb (GtkAction *action, ActionData *action_data)
-{
-  vinagre_vnc_listener_dialog_show (action_data->window, action_data->plugin);
-}
-
-static GtkActionEntry action_entries[] =
-{
-  { "VNCListener",
-    NULL,
-    /* Translators: "Reverse" here is an adjective, not a verb. */
-    N_("_Reverse Connections..."),
-    NULL,
-    N_("Configure incoming VNC connections"),
-    G_CALLBACK (listening_cb)
-  }
-};
-
-static void
-impl_activate (VinagrePlugin *plugin,
-               VinagreWindow *window)
-{
-  GtkActionGroup *action_group;
-  GtkUIManager *manager;
-  WindowData *data;
-  ActionData *action_data;
-  gboolean always;
-
-  vinagre_debug_message (DEBUG_PLUGINS, "VinagreVncPlugin Activate");
-
-  data = g_slice_new (WindowData);
-  action_data = g_slice_new (ActionData);
-  action_data->window = window;
-  action_data->plugin = plugin;
-
-  action_group = vinagre_window_get_always_sensitive_action (window);
-  manager = vinagre_window_get_ui_manager (window);
-
-  data->ui_action_group = gtk_action_group_new ("VinagreVNCPluginActions");
-  gtk_action_group_set_translation_domain (data->ui_action_group, GETTEXT_PACKAGE);
-  gtk_action_group_add_actions_full (data->ui_action_group,
-				     action_entries,
-				     G_N_ELEMENTS (action_entries),
-				     action_data,
-				     (GDestroyNotify) free_action_data);
-  gtk_ui_manager_insert_action_group (manager,
-				      data->ui_action_group,
-				      -1);
-
-  data->ui_id = gtk_ui_manager_new_merge_id (manager);
-  gtk_ui_manager_add_ui (manager,
-			 data->ui_id,
-			 "/MenuBar/MachineMenu/MachineOps_1",
-			 "VNCListener",
-			 "VNCListener",
-			 GTK_UI_MANAGER_AUTO,
-			 TRUE);
-
-  g_object_set_data_full (G_OBJECT (window),
-			  WINDOW_DATA_KEY,
-			  data,
-			  (GDestroyNotify) free_window_data);
-
-  g_object_get (vinagre_prefs_get_default (),
-		"always-enable-listening", &always,
-		NULL);
-  if (always)
-    vinagre_vnc_listener_start (vinagre_vnc_listener_get_default ());
-}
-
-static void
-impl_deactivate  (VinagrePlugin *plugin,
-                  VinagreWindow *window)
-{
-  GtkUIManager *manager;
-  WindowData *data;
-
-  vinagre_debug_message (DEBUG_PLUGINS, "VinagreVncPlugin Deactivate");
-
-  manager = vinagre_window_get_ui_manager (window);
-  data = (WindowData *) g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
-  g_return_if_fail (data != NULL);
-
-  gtk_ui_manager_remove_ui (manager, data->ui_id);
-  gtk_ui_manager_remove_action_group (manager, data->ui_action_group);
-
-  g_object_set_data (G_OBJECT (window), WINDOW_DATA_KEY, NULL);
-}
-
-static void
-impl_update_ui (VinagrePlugin *plugin,
-                VinagreWindow *window)
-{
-  vinagre_debug_message (DEBUG_PLUGINS, "VinagreVncPlugin Update UI");
-}
+static void vinagre_vnc_protocol_iface_init (VinagreProtocolInterface *iface);
+G_DEFINE_DYNAMIC_TYPE_EXTENDED (VinagreVncPlugin,
+				vinagre_vnc_plugin,
+				PEAS_TYPE_EXTENSION_BASE,
+				0,
+				G_IMPLEMENT_INTERFACE_DYNAMIC (VINAGRE_TYPE_PROTOCOL,
+							       vinagre_vnc_protocol_iface_init))
 
 static const GOptionEntry vinagre_vnc_args[] =
 {
@@ -176,13 +52,11 @@ static const GOptionEntry vinagre_vnc_args[] =
 };
 
 static GSList *
-impl_get_context_groups (VinagrePlugin *plugin)
+impl_get_context_groups (VinagreProtocol *plugin)
 {
   GOptionGroup *group;
   GSList       *groups = NULL;
 
-  vinagre_debug_message (DEBUG_PLUGINS, "VinagreVncPlugin Get Context Group");
-
   scaling_command_line = FALSE;
   group = g_option_group_new ("vnc",
 			      /* Translators: this is a command line option (run vinagre --help) */
@@ -200,13 +74,13 @@ impl_get_context_groups (VinagrePlugin *plugin)
 }
 
 static const gchar *
-impl_get_protocol (VinagrePlugin *plugin)
+impl_get_protocol (VinagreProtocol *plugin)
 {
   return "vnc";
 }
 
 static gchar **
-impl_get_public_description (VinagrePlugin *plugin)
+impl_get_public_description (VinagreProtocol *plugin)
 {
   gchar **result = g_new (gchar *, 3);
 
@@ -218,13 +92,13 @@ impl_get_public_description (VinagrePlugin *plugin)
 }
 
 static const gchar *
-impl_get_mdns_service (VinagrePlugin *plugin)
+impl_get_mdns_service (VinagreProtocol *plugin)
 {
   return "_rfb._tcp";
 }
 
 static VinagreConnection *
-impl_new_connection (VinagrePlugin *plugin)
+impl_new_connection (VinagreProtocol *plugin)
 {
   VinagreConnection *conn;
 
@@ -236,7 +110,7 @@ impl_new_connection (VinagrePlugin *plugin)
 }
 
 static VinagreConnection *
-impl_new_connection_from_file (VinagrePlugin *plugin,
+impl_new_connection_from_file (VinagreProtocol *plugin,
 			       const gchar   *data,
 			       gboolean       use_bookmarks,
 			       gchar        **error_msg)
@@ -335,7 +209,7 @@ the_end:
 }
 
 static GtkWidget *
-impl_new_tab (VinagrePlugin *plugin,
+impl_new_tab (VinagreProtocol *plugin,
 	      VinagreConnection *conn,
 	      VinagreWindow     *window)
 {
@@ -366,7 +240,7 @@ scaling_check_toggled_cb (GtkToggleButton *button, GObject *box)
 }
 
 static GtkWidget *
-impl_get_connect_widget (VinagrePlugin *plugin, VinagreConnection *conn)
+impl_get_connect_widget (VinagreProtocol *plugin, VinagreConnection *conn)
 {
   GtkWidget *box, *check, *label, *combo, *box2, *ssh_host_entry;
   GtkTable  *table;
@@ -436,12 +310,12 @@ impl_get_connect_widget (VinagrePlugin *plugin, VinagreConnection *conn)
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_box_pack_start (GTK_BOX (box2), GTK_WIDGET (label), FALSE, FALSE, 0);
 
-  combo = gtk_combo_box_new_text ();
-  gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Use Server Settings"));
-  gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("True Color (24 bits)"));
-  gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("High Color (16 bits)"));
-  gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Low Color (8 bits)"));
-  gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Ultra Low Color (3 bits)"));
+  combo = gtk_combo_box_text_new ();
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Use Server Settings"));
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("True Color (24 bits)"));
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("High Color (16 bits)"));
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Low Color (8 bits)"));
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Ultra Low Color (3 bits)"));
   gtk_combo_box_set_active (GTK_COMBO_BOX (combo),
 			    has_conn ? vinagre_vnc_connection_get_depth_profile (VINAGRE_VNC_CONNECTION (conn))
 			    : vinagre_cache_prefs_get_integer ("vnc-connection", "depth-profile", 0));
@@ -462,7 +336,12 @@ impl_get_connect_widget (VinagrePlugin *plugin, VinagreConnection *conn)
   gtk_widget_set_sensitive (ssh_host_entry, FALSE);
   g_object_set_data (G_OBJECT (box), "ssh_host", ssh_host_entry);
   /* Translators: This is the tooltip of the SSH tunneling entry */
-  gtk_widget_set_tooltip_text (ssh_host_entry, _("hostname or user@hostname"));
+  str = g_strdup_printf ("%s\n%s\n%s",
+			 _("hostname or user@hostname"),
+			 _("Supply an alternative port using colon"),
+			 _("For instance: joe@domain.com:5022"));
+  gtk_widget_set_tooltip_text (ssh_host_entry, str);
+  g_free (str);
   gtk_box_pack_start (GTK_BOX (box2), ssh_host_entry, FALSE, FALSE, 0);
 
   /* Translators: the whole sentence will be: Use host <hostname> as a SSH tunnel*/
@@ -489,13 +368,13 @@ impl_get_connect_widget (VinagrePlugin *plugin, VinagreConnection *conn)
 }
 
 static gint
-impl_get_default_port (VinagrePlugin *plugin)
+impl_get_default_port (VinagreProtocol *plugin)
 {
   return 5900;
 }
 
 static GtkFileFilter *
-impl_get_file_filter (VinagrePlugin *plugin)
+impl_get_file_filter (VinagreProtocol *plugin)
 {
   GtkFileFilter *filter;
 
@@ -507,40 +386,41 @@ impl_get_file_filter (VinagrePlugin *plugin)
   return filter;
 }
 
-static void
-vinagre_vnc_plugin_init (VinagreVncPlugin *plugin)
-{
-  vinagre_debug_message (DEBUG_PLUGINS, "VinagreVncPlugin initializing");
-}
-
-static void
-vinagre_vnc_plugin_finalize (GObject *object)
-{
-  vinagre_debug_message (DEBUG_PLUGINS, "VinagreVncPlugin finalizing");
-
-  G_OBJECT_CLASS (vinagre_vnc_plugin_parent_class)->finalize (object);
-}
-
 static void
 vinagre_vnc_plugin_class_init (VinagreVncPluginClass *klass)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  VinagrePluginClass *plugin_class = VINAGRE_PLUGIN_CLASS (klass);
-
-  object_class->finalize   = vinagre_vnc_plugin_finalize;
-
-  plugin_class->activate   = impl_activate;
-  plugin_class->deactivate = impl_deactivate;
-  plugin_class->update_ui  = impl_update_ui;
-  plugin_class->get_context_groups = impl_get_context_groups;
-  plugin_class->get_protocol  = impl_get_protocol;
-  plugin_class->get_public_description  = impl_get_public_description;
-  plugin_class->new_connection = impl_new_connection;
-  plugin_class->new_connection_from_file = impl_new_connection_from_file;
-  plugin_class->get_mdns_service  = impl_get_mdns_service;
-  plugin_class->new_tab = impl_new_tab;
-  plugin_class->get_connect_widget = impl_get_connect_widget;
-  plugin_class->get_default_port = impl_get_default_port;
-  plugin_class->get_file_filter = impl_get_file_filter;
 }
+static void
+vinagre_vnc_plugin_class_finalize (VinagreVncPluginClass *klass)
+{
+}
+static void
+vinagre_vnc_plugin_init (VinagreVncPlugin *plugin)
+{
+}
+
+static void
+vinagre_vnc_protocol_iface_init (VinagreProtocolInterface *iface)
+{
+  iface->get_context_groups = impl_get_context_groups;
+  iface->get_protocol  = impl_get_protocol;
+  iface->get_public_description  = impl_get_public_description;
+  iface->new_connection = impl_new_connection;
+  iface->new_connection_from_file = impl_new_connection_from_file;
+  iface->get_mdns_service  = impl_get_mdns_service;
+  iface->new_tab = impl_new_tab;
+  iface->get_connect_widget = impl_get_connect_widget;
+  iface->get_default_port = impl_get_default_port;
+  iface->get_file_filter = impl_get_file_filter;
+}
+
+G_MODULE_EXPORT void
+peas_register_types (PeasObjectModule *module)
+{
+  vinagre_vnc_plugin_register_type (G_TYPE_MODULE (module));
+  peas_object_module_register_extension_type (module,
+					      VINAGRE_TYPE_PROTOCOL,
+					      VINAGRE_TYPE_VNC_PLUGIN);
+}
+
 /* vim: set ts=8: */
